{% extends "bases/bootstrap.html" %}
{% load staticfiles %}

{% block tail_js %}

<script src="//ajax.googleapis.com/ajax/libs/jquery/1.11.0/jquery.min.js"></script>
<!-- <script>window.jQuery || document.write('<script src="js/jquery-1.10.1.min.js"><\/script>')</script> -->
<script src="//netdna.bootstrapcdn.com/bootstrap/3.1.1/js/bootstrap.min.js"></script>

{% block app_js %}
{% endblock %}

{% endblock %}

