{% extends "bases/bootstrap-jquery.html" %}
{% load staticfiles %}

{% block nav %}
    <!-- Fixed navbar -->
    <div class="navbar navbar-default navbar-static-top navbar-shadow" role="navigation">
      <div class="container">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="{% url 'user_home' %}">Enough Community Member</a>
        </div>
        <div class="navbar-collapse collapse">
          <ul class="nav navbar-nav navbar-right">
            <li><a href="{% url 'landing_index' %}">Home</a></li>
            <li><a href="{% url 'account_logout' %}"></i> Sign Out</a></li>
            <li>{% if user.profile.avatar_url %}<img alt="" style="width:50px; height:50px" src="{{user.profile.avatar_url}}">{% endif %}</li>

          </ul>
        </div>
      </div>
    </div>

{% endblock %}

