{% extends "bases/bootstrap-visitor.html" %}

{% block head_title %}Social Network Login Failure{% endblock %}

{% block content %}
<h1>Social Network Login Failure</h1>

<p>An error occurred while attempting to login via your social network account.</p>
<p>{{ auth_error }}</p>
{% endblock %}
