{% extends "bases/bootstrap-visitor.html" %}
{% load staticfiles %}

{% block content %}

<div class="container">
  <div class="jumbotron text-center">
    <h1>Fedeproxy API</h1>
  </div>
</div>

{% endblock %}
