{% load staticfiles %}
<!DOCTYPE html>

<html lang="en" {% block htmlattr %}{% endblock %}>
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <title>{% block title %}allauth demo{% endblock %}</title>

  <link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css">
  <link rel="stylesheet" href="//netdna.bootstrapcdn.com/font-awesome/4.1.0/css/font-awesome.css">

  <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
  <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
  <!--[if lt IE 9]>
  <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
  <script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
  <![endif]-->

  {% block head_css %}
  <link rel="stylesheet" media="screen" href='{% static "css/main.css" %}'>
  {% endblock %}

</head>
<body class='{% block pageclass %}{% endblock %}' {% block bodyattr %}{% endblock %}>

{% block nav %}{% endblock %}

{% block content %}{% endblock %}

{% block footer %}
<div class=container>
  <hr>
  <footer>
  </footer>
</div>
{% endblock %}

{% block tail_js %}
{% endblock %}

</body>
</html>

